@@ -1,23 +1,24 @@
-use ::syn::{self, Field, VariantData, Ident};
-use ::quote::{self, Tokens};
+use proc_macro2::{Span, TokenStream};
 
-pub fn transcoder_decorator(ast: &syn::MacroInput) -> Result<quote::Tokens, &str> {
-    let struct_fields = match ast.body {
-        syn::Body::Struct(VariantData::Struct(ref data)) => data,
-        _ => return Err("Must decorate a struct"),
+use ::syn::{self, Data, Field, Ident};
+
+pub fn transcoder_decorator(ast: &syn::DeriveInput) -> TokenStream {
+    let struct_fields = match ast.data {
+        Data::Struct(ref data_struct) => &data_struct.fields,
+        _ => panic!("Must decorate a struct"),
     };
 
-    let num_lifetimes = ast.generics.lifetimes.len();
+    let num_lifetimes = ast.generics.lifetimes().count();
     if num_lifetimes > 1 {
-        return Err("Struct can only have one lifetime argument");
+        panic!("Struct can only have one lifetime argument");
     }
     let has_lifetime = num_lifetimes == 1;
 
-    let field_atoms: Vec<Tokens> = struct_fields.iter().map(|field| {
-        let ident = field.clone().ident.unwrap();
+    let field_atoms: Vec<TokenStream> = struct_fields.iter().map(|field| {
+        let ident = field.ident.as_ref().unwrap();
         let ident_str = ident.to_string();
 
-        let atom_fun = Ident::new(format!("atom_{}", ident_str));
+        let atom_fun = Ident::new(&format!("atom_{}", ident_str), Span::call_site());
 
         quote! {
             atom #atom_fun = #ident_str;
@@ -29,21 +30,25 @@ pub fn transcoder_decorator(ast: &syn::MacroInput) -> Result<quote::Tokens, &str
         }
     };
 
-    let decoder = gen_decoder(&ast.ident, struct_fields, &atom_defs, has_lifetime);
-    let encoder = gen_encoder(&ast.ident, struct_fields, &atom_defs, has_lifetime);
+    let struct_fields: Vec<_> = struct_fields.iter().collect();
 
-    Ok(quote! {
+    let decoder = gen_decoder(&ast.ident, &struct_fields, &atom_defs, has_lifetime);
+    let encoder = gen_encoder(&ast.ident, &struct_fields, &atom_defs, has_lifetime);
+
+    let gen = quote! {
         #decoder
         #encoder
-    })
+    };
+
+    gen.into()
 }
 
-pub fn gen_decoder(struct_name: &Ident, fields: &Vec<Field>, atom_defs: &Tokens, has_lifetime: bool) -> Tokens {
-    let field_defs: Vec<Tokens> = fields.iter().map(|field| {
-        let ident = field.clone().ident.unwrap();
+pub fn gen_decoder(struct_name: &Ident, fields: &[&Field], atom_defs: &TokenStream, has_lifetime: bool) -> TokenStream {
+    let field_defs: Vec<TokenStream> = fields.iter().map(|field| {
+        let ident = field.ident.as_ref().unwrap();
         let ident_str = ident.to_string();
 
-        let atom_fun = Ident::new(format!("atom_{}", ident_str));
+        let atom_fun = Ident::new(&format!("atom_{}", ident_str), Span::call_site());
 
         quote! {
             #ident: ::rustler::Decoder::decode(term.map_get(#atom_fun().encode(env))?)?
@@ -56,7 +61,7 @@ pub fn gen_decoder(struct_name: &Ident, fields: &Vec<Field>, atom_defs: &Tokens,
         quote! { #struct_name }
     };
 
-    quote! {
+    let gen = quote! {
         impl<'a> ::rustler::Decoder<'a> for #struct_type {
             fn decode(term: ::rustler::Term<'a>) -> Result<Self, ::rustler::Error> {
                 #atom_defs
@@ -65,15 +70,17 @@ pub fn gen_decoder(struct_name: &Ident, fields: &Vec<Field>, atom_defs: &Tokens,
                 Ok(#struct_name { #(#field_defs),* })
             }
         }
-    }
+    };
+
+    gen.into()
 }
 
-pub fn gen_encoder(struct_name: &Ident, fields: &Vec<Field>, atom_defs: &Tokens, has_lifetime: bool) -> Tokens {
-    let field_defs: Vec<Tokens> = fields.iter().map(|field| {
-        let field_ident = field.clone().ident.unwrap();
+pub fn gen_encoder(struct_name: &Ident, fields: &[&Field], atom_defs: &TokenStream, has_lifetime: bool) -> TokenStream {
+    let field_defs: Vec<TokenStream> = fields.iter().map(|field| {
+        let field_ident = field.ident.as_ref().unwrap();
         let field_ident_str = field_ident.to_string();
 
-        let atom_fun = Ident::new(format!("atom_{}", field_ident_str));
+        let atom_fun = Ident::new(&format!("atom_{}", field_ident_str), Span::call_site());
 
         quote! {
             map = map.map_put(#atom_fun().encode(env), self.#field_ident.encode(env)).ok().unwrap();
@@ -86,7 +93,7 @@ pub fn gen_encoder(struct_name: &Ident, fields: &Vec<Field>, atom_defs: &Tokens,
         quote! { #struct_name }
     };
 
-    quote! {
+    let gen = quote! {
         impl<'b> ::rustler::Encoder for #struct_type {
             fn encode<'a>(&self, env: ::rustler::Env<'a>) -> ::rustler::Term<'a> {
                 #atom_defs
@@ -96,5 +103,7 @@ pub fn gen_encoder(struct_name: &Ident, fields: &Vec<Field>, atom_defs: &Tokens,
                 map
             }
         }
-    }
+    };
+
+    gen.into()
 }
