@@ -1,26 +1,36 @@
-use ::syn::{self, Field, VariantData, MetaItem, Lit, Ident};
-use ::quote::{self, Tokens};
+use proc_macro2::TokenStream;
 
-pub fn transcoder_decorator(ast: &syn::MacroInput) -> Result<quote::Tokens, &str> {
-    let record_tag = {
+use ::syn::{self, Data, Field, Meta, Ident, Lit};
+
+pub fn transcoder_decorator(ast: &syn::DeriveInput) -> TokenStream {
+    let record_tag: String = {
         let ref attr_value = ast.attrs.iter()
-            .find(|attr| attr.name() == "tag")
-            .expect("NifRecord requires a 'tag' attribute").value;
-        assert!(attr_value.name() == "tag", "NifRecord requires a 'tag' attribute");
+            .map(|attr| attr.parse_meta())
+            .find(|meta| match meta {
+                Ok(Meta::NameValue(meta_name_value)) =>
+                    meta_name_value.ident == "tag",
+                _ => false
+            })
+            .expect("NifRecord requires a 'tag' attribute");
+
         match *attr_value {
-            MetaItem::NameValue(_, Lit::Str(ref value, _)) => value,
+            Ok(Meta::NameValue(ref meta_name_value)) => match meta_name_value.lit {
+                Lit::Str(ref tag) => tag.value(),
+                _ => panic!("Cannot parse tag")
+            }
             _ => panic!("NifRecord requires a 'tag' attribute"),
         }
     };
 
-    let struct_fields = match ast.body {
-        syn::Body::Struct(VariantData::Struct(ref data)) => data,
-        _ => return Err("Must decorate a struct"),
+    let struct_fields = match ast.data {
+        Data::Struct(ref data_struct) => &data_struct.fields,
+        Data::Enum(_) => panic!("NifRecord can only be used with structs"),
+        Data::Union(_) => panic!("NifRecord can only be used with enums"),
     };
 
-    let num_lifetimes = ast.generics.lifetimes.len();
+    let num_lifetimes = ast.generics.lifetimes().count();
     if num_lifetimes > 1 {
-        return Err("Struct can only have one lifetime argument");
+        panic!("Struct can only have one lifetime argument");
     }
     let has_lifetime = num_lifetimes == 1;
 
@@ -30,21 +40,25 @@ pub fn transcoder_decorator(ast: &syn::MacroInput) -> Result<quote::Tokens, &str
         }
     };
 
-    let decoder = gen_decoder(&ast.ident, struct_fields, &atom_defs, has_lifetime);
-    let encoder = gen_encoder(&ast.ident, struct_fields, &atom_defs, has_lifetime);
+    let struct_fields: Vec<_> = struct_fields.iter().collect();
 
-    Ok(quote! {
+    let decoder = gen_decoder(&ast.ident, &struct_fields, &atom_defs, has_lifetime);
+    let encoder = gen_encoder(&ast.ident, &struct_fields, &atom_defs, has_lifetime);
+
+    let gen = quote! {
         #decoder
         #encoder
-    })
+    };
+
+    gen.into()
 }
 
-pub fn gen_decoder(struct_name: &Ident, fields: &Vec<Field>, atom_defs: &Tokens, has_lifetime: bool) -> Tokens {
+pub fn gen_decoder(struct_name: &Ident, fields: &[&Field], atom_defs: &TokenStream, has_lifetime: bool) -> TokenStream {
     // Make a decoder for each of the fields in the struct.
-    let field_defs: Vec<Tokens> = fields.iter().enumerate().map(|(idx, field)| {
-        let decoder = quote! { try!(::rustler::Decoder::decode(terms[#idx + 1])) };
+    let field_defs: Vec<TokenStream> = fields.iter().enumerate().map(|(idx, field)| {
+        let decoder = quote! { ::rustler::Decoder::decode(terms[#idx + 1])? };
 
-        let ident = field.clone().ident.unwrap();
+        let ident = field.ident.as_ref().unwrap();
         quote! { #ident: #decoder }
     }).collect();
 
@@ -58,10 +72,10 @@ pub fn gen_decoder(struct_name: &Ident, fields: &Vec<Field>, atom_defs: &Tokens,
     let field_num = field_defs.len();
 
     // The implementation itself
-    quote! {
+    let gen = quote! {
         impl<'a> ::rustler::Decoder<'a> for #struct_typ {
             fn decode(term: ::rustler::Term<'a>) -> Result<Self, ::rustler::Error> {
-                let terms = try!(::rustler::types::tuple::get_tuple(term));
+                let terms = ::rustler::types::tuple::get_tuple(term)?;
                 if terms.len() != #field_num + 1 {
                     return Err(::rustler::Error::Atom("invalid_record"));
                 }
@@ -80,13 +94,15 @@ pub fn gen_decoder(struct_name: &Ident, fields: &Vec<Field>, atom_defs: &Tokens,
                 )
             }
         }
-    }
+    };
+
+    gen.into()
 }
 
-pub fn gen_encoder(struct_name: &Ident, fields: &Vec<Field>, atom_defs: &Tokens, has_lifetime: bool) -> Tokens {
+pub fn gen_encoder(struct_name: &Ident, fields: &[&Field], atom_defs: &TokenStream, has_lifetime: bool) -> TokenStream {
     // Make a field encoder expression for each of the items in the struct.
-    let field_encoders: Vec<Tokens> = fields.iter().map(|field| {
-        let field_ident = field.clone().ident.unwrap();
+    let field_encoders: Vec<TokenStream> = fields.iter().map(|field| {
+        let field_ident = field.ident.as_ref().unwrap();
         let field_source = quote! { self.#field_ident };
         quote! { #field_source.encode(env) }
     }).collect();
@@ -107,7 +123,7 @@ pub fn gen_encoder(struct_name: &Ident, fields: &Vec<Field>, atom_defs: &Tokens,
     };
 
     // The implementation itself
-    quote! {
+    let gen = quote! {
         impl<'b> ::rustler::Encoder for #struct_typ {
             fn encode<'a>(&self, env: ::rustler::Env<'a>) -> ::rustler::Term<'a> {
                 #atom_defs
@@ -117,5 +133,7 @@ pub fn gen_encoder(struct_name: &Ident, fields: &Vec<Field>, atom_defs: &Tokens,
                 ::rustler::types::tuple::make_tuple(env, &arr)
             }
         }
-    }
+    };
+
+    gen.into()
 }
