@@ -1,26 +1,29 @@
-use heck::SnakeCase;
-use syn::{self, Body, Ident, Variant, VariantData};
-use quote::{self, Tokens};
+use proc_macro2::{Span, TokenStream};
 
-pub fn transcoder_decorator(ast: &syn::DeriveInput) -> Result<quote::Tokens, &str> {
-    let variants = match ast.body {
-        Body::Enum(ref variants) => variants,
-        Body::Struct(_) => panic!("NifUnitEnum can only be used with enums"),
+use heck::SnakeCase;
+use syn::{self, Data, Ident, Variant, Fields};
+
+pub fn transcoder_decorator(ast: &syn::DeriveInput) -> TokenStream {
+    let variants = match ast.data {
+        Data::Enum(ref data_enum) => &data_enum.variants,
+        Data::Struct(_) => panic!("NifUnitEnum can only be used with enums"),
+        Data::Union(_) => panic!("NifUnitEnum can only be used with enums"),
     };
 
-    let num_lifetimes = ast.generics.lifetimes.len();
+    let num_lifetimes = ast.generics.lifetimes().count();
     if num_lifetimes > 1 { panic!("Enum can only have one lifetime argument"); }
     let has_lifetime = num_lifetimes == 1;
 
     for variant in variants {
-        if VariantData::Unit != variant.data {
+        if let Fields::Unit = variant.fields {
+        } else {
             panic!("NifUnitEnum can only be used with enums that contain all unit variants.");
         }
     }
 
-    let atoms: Vec<Tokens> = variants.iter().map(|variant| {
+    let atoms: Vec<TokenStream> = variants.iter().map(|variant| {
         let atom_str = variant.ident.to_string().to_snake_case();
-        let atom_fn  = Ident::new(format!("atom_{}", atom_str));
+        let atom_fn  = Ident::new(&format!("atom_{}", atom_str), Span::call_site());
         quote! {
             atom #atom_fn = #atom_str;
         }
@@ -32,26 +35,30 @@ pub fn transcoder_decorator(ast: &syn::DeriveInput) -> Result<quote::Tokens, &st
         }
     };
 
-    let decoder = gen_decoder(&ast.ident, variants, &atom_defs, has_lifetime);
-    let encoder = gen_encoder(&ast.ident, variants, &atom_defs, has_lifetime);
+    let variants: Vec<&Variant> = variants.iter().collect();
 
-    Ok(quote! {
+    let decoder = gen_decoder(&ast.ident, &variants, &atom_defs, has_lifetime);
+    let encoder = gen_encoder(&ast.ident, &variants, &atom_defs, has_lifetime);
+
+    let gen = quote! {
         #decoder
         #encoder
-    })
+    };
+
+    gen.into()
 }
 
-pub fn gen_decoder(enum_name: &Ident, variants: &[Variant], atom_defs: &Tokens, has_lifetime: bool) -> Tokens {
+pub fn gen_decoder(enum_name: &Ident, variants: &[&Variant], atom_defs: &TokenStream, has_lifetime: bool) -> TokenStream {
     let enum_type = if has_lifetime {
         quote! { #enum_name <'b> }
     } else {
         quote! { #enum_name }
     };
 
-    let variant_defs: Vec<Tokens> = variants.iter().map(|variant| {
-        let variant_ident = variant.ident.clone();
+    let variant_defs: Vec<TokenStream> = variants.iter().map(|variant| {
+        let variant_ident = &variant.ident;
         let atom_str      = variant_ident.to_string().to_snake_case();
-        let atom_fn       = Ident::new(format!("atom_{}", atom_str));
+        let atom_fn       = Ident::new(&format!("atom_{}", atom_str), Span::call_site());
 
         quote! {
             if value == #atom_fn() {
@@ -60,7 +67,7 @@ pub fn gen_decoder(enum_name: &Ident, variants: &[Variant], atom_defs: &Tokens,
         }
     }).collect();
 
-    quote! {
+    let gen = quote! {
         impl<'a> ::rustler::Decoder<'a> for #enum_type {
             fn decode(term: ::rustler::Term<'a>) -> Result<Self, ::rustler::Error> {
                 #atom_defs
@@ -72,27 +79,29 @@ pub fn gen_decoder(enum_name: &Ident, variants: &[Variant], atom_defs: &Tokens,
                 Err(::rustler::Error::Atom("invalid_variant"))
             }
         }
-    }
+    };
+
+    gen.into()
 }
 
-pub fn gen_encoder(enum_name: &Ident, variants: &[Variant], atom_defs: &Tokens, has_lifetime: bool) -> Tokens {
+pub fn gen_encoder(enum_name: &Ident, variants: &[&Variant], atom_defs: &TokenStream, has_lifetime: bool) -> TokenStream {
     let enum_type = if has_lifetime {
         quote! { #enum_name <'b> }
     } else {
         quote! { #enum_name }
     };
 
-    let variant_defs: Vec<Tokens> = variants.iter().map(|variant| {
-        let variant_ident = variant.ident.clone();
+    let variant_defs: Vec<TokenStream> = variants.iter().map(|variant| {
+        let variant_ident = &variant.ident;
         let atom_str      = variant_ident.to_string().to_snake_case();
-        let atom_fn       = Ident::new(format!("atom_{}", atom_str));
+        let atom_fn       = Ident::new(&format!("atom_{}", atom_str), Span::call_site());
 
         quote! {
             #enum_name :: #variant_ident => #atom_fn().encode(env),
         }
     }).collect();
 
-    quote! {
+    let gen = quote! {
         impl<'b> ::rustler::Encoder for #enum_type {
             fn encode<'a>(&self, env: ::rustler::Env<'a>) -> ::rustler::Term<'a> {
                 #atom_defs
@@ -102,5 +111,7 @@ pub fn gen_encoder(enum_name: &Ident, variants: &[Variant], atom_defs: &Tokens,
                 }
             }
         }
-    }
+    };
+
+    gen.into()
 }
