@@ -1,19 +1,21 @@
-use ::syn::{self, Body, Ident, Variant, VariantData};
-use ::quote::{self, Tokens};
+use proc_macro2::TokenStream;
 
-pub fn transcoder_decorator(ast: &syn::DeriveInput) -> Result<quote::Tokens, &str> {
-    let variants = match ast.body {
-        Body::Enum(ref variants) => variants,
-        Body::Struct(_) => panic!("NifUntaggedEnum can only be used with enums"),
+use ::syn::{self, Data, Ident, Variant, Fields};
+
+pub fn transcoder_decorator(ast: &syn::DeriveInput) -> TokenStream {
+    let variants = match ast.data {
+        Data::Enum(ref data_enum) => &data_enum.variants,
+        Data::Struct(_) => panic!("NifUntaggedEnum can only be used with enums"),
+        Data::Union(_) => panic!("NifUntaggedEnum can only be used with enums"),
     };
 
-    let num_lifetimes = ast.generics.lifetimes.len();
+    let num_lifetimes = ast.generics.lifetimes().count();
     if num_lifetimes > 1 { panic!("Enum can only have one lifetime argument"); }
     let has_lifetime = num_lifetimes == 1;
 
     for variant in variants {
-        if let VariantData::Tuple(ref fields) = variant.data {
-            if fields.len() != 1 {
+        if let Fields::Unnamed(_) = variant.fields {
+            if variant.fields.iter().count() != 1 {
                 panic!("NifUntaggedEnum can only be used with enums that contain all NewType variants.");
             }
         } else {
@@ -21,25 +23,29 @@ pub fn transcoder_decorator(ast: &syn::DeriveInput) -> Result<quote::Tokens, &st
         }
     }
 
-    let decoder = gen_decoder(&ast.ident, variants, has_lifetime);
-    let encoder = gen_encoder(&ast.ident, variants, has_lifetime);
+    let variants: Vec<&Variant> = variants.iter().collect();
 
-    Ok(quote! {
+    let decoder = gen_decoder(&ast.ident, &variants, has_lifetime);
+    let encoder = gen_encoder(&ast.ident, &variants, has_lifetime);
+
+    let gen = quote!{
         #decoder
         #encoder
-    })
+    };
+
+    gen.into()
 }
 
-pub fn gen_decoder(enum_name: &Ident, variants: &[Variant], has_lifetime: bool) -> Tokens {
+pub fn gen_decoder(enum_name: &Ident, variants: &[&Variant], has_lifetime: bool) -> TokenStream {
     let enum_type = if has_lifetime {
         quote! { #enum_name <'b> }
     } else {
         quote! { #enum_name }
     };
 
-    let variant_defs: Vec<Tokens> = variants.iter().map(|variant| {
-        let variant_name = variant.ident.clone();
-        let field_type   = variant.data.fields()[0].ty.clone();
+    let variant_defs: Vec<_> = variants.iter().map(|variant| {
+        let variant_name = &variant.ident;
+        let field_type   = &variant.fields.iter().next().unwrap().ty;
 
         quote! {
             if let Ok(inner) = #field_type::decode(term) {
@@ -48,7 +54,7 @@ pub fn gen_decoder(enum_name: &Ident, variants: &[Variant], has_lifetime: bool)
         }
     }).collect();
 
-    quote! {
+    let gen = quote! {
         impl<'a> ::rustler::Decoder<'a> for #enum_type {
             fn decode(term: ::rustler::Term<'a>) -> Result<Self, ::rustler::Error> {
                 #(#variant_defs)*
@@ -56,25 +62,27 @@ pub fn gen_decoder(enum_name: &Ident, variants: &[Variant], has_lifetime: bool)
                 Err(::rustler::Error::Atom("invalid_variant"))
             }
         }
-    }
+    };
+
+    gen.into()
 }
 
-pub fn gen_encoder(enum_name: &Ident, variants: &[Variant], has_lifetime: bool) -> Tokens {
+pub fn gen_encoder(enum_name: &Ident, variants: &[&Variant], has_lifetime: bool) -> TokenStream {
     let enum_type = if has_lifetime {
         quote! { #enum_name <'b> }
     } else {
         quote! { #enum_name }
     };
 
-    let variant_defs: Vec<Tokens> = variants.iter().map(|variant| {
-        let variant_name = variant.ident.clone();
+    let variant_defs: Vec<_> = variants.iter().map(|variant| {
+        let variant_name = &variant.ident;
 
         quote! {
             #enum_name :: #variant_name ( ref inner ) => inner.encode(env),
         }
     }).collect();
 
-    quote! {
+    let gen = quote! {
         impl<'b> ::rustler::Encoder for #enum_type {
             fn encode<'a>(&self, env: ::rustler::Env<'a>) -> ::rustler::Term<'a> {
                 match *self {
@@ -82,5 +90,7 @@ pub fn gen_encoder(enum_name: &Ident, variants: &[Variant], has_lifetime: bool)
                 }
             }
         }
-    }
+    };
+
+    gen.into()
 }
